﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.IO;

namespace TestPODll
{
    static class PngOptimizerDll
    {
        public enum POChunkOption : uint
        {
            Remove,
            Keep,
            Force
        };

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        public struct POSettings
        {
            public bool BackupOldPngFiles;
            public bool KeepInterlacing;
            public bool IE6Compatible;
            public bool SkipAnimatedGifs;
            public bool KeepFileDate;

            public POChunkOption BkgdOption;
            public int BkgdColor;      // Forced color

            public POChunkOption TextOption;
            public string TextKeyword; // Forced text keyword
            public string TextData;    // Forced text data

            public POChunkOption PhysOption;
            public uint PhysPpmX;      // Forced Pixels per meter X
            public uint PhysPpmY;      // Forced Pixels per meter Y
        };

        [DllImport(@"..\PngOptimizerDll.dll", EntryPoint = "PO_OptimizeFile")]
        public static extern bool OptimizeFile([MarshalAs(UnmanagedType.LPWStr)]string filePath);

        [DllImport(@"..\PngOptimizerDll.dll", EntryPoint = "PO_OptimizeFileMem")]
        public static extern bool OptimizeFileMem(byte[] image, int imageSize,
                                                  [Out] byte[] result, int resultCapacity, out int resultSize);

        [DllImport(@"..\PngOptimizerDll.dll", EntryPoint = "PO_GetLastErrorString")]
        [return: MarshalAs(UnmanagedType.LPWStr)]
        public static extern string GetLastErrorString();

        [DllImport(@"..\PngOptimizerDll.dll", EntryPoint = "PO_SetSettings")]
        public static extern bool SetSettings(POSettings settings);

        [DllImport(@"..\PngOptimizerDll.dll", EntryPoint = "PO_GetSettings")]
        public static extern bool GetSettings(out POSettings settings);
    }

    class TestPODllApp
    {
        public static bool AreSettingsEqual(PngOptimizerDll.POSettings s1, PngOptimizerDll.POSettings s2)
        {
            bool eq = s1.BackupOldPngFiles == s2.BackupOldPngFiles;
            eq &= s1.KeepInterlacing == s2.KeepInterlacing;
            eq &= s1.IE6Compatible == s2.IE6Compatible;
            eq &= s1.SkipAnimatedGifs == s2.SkipAnimatedGifs;
            eq &= s1.KeepFileDate == s2.KeepFileDate;
            eq &= s1.BkgdOption == s2.BkgdOption;
            eq &= s1.BkgdColor == s2.BkgdColor;
            eq &= s1.TextOption == s2.TextOption;
            eq &= s1.TextKeyword == s2.TextKeyword;
            eq &= s1.TextData == s2.TextData;
            eq &= s1.PhysOption == s2.PhysOption;
            eq &= s1.PhysPpmX == s2.PhysPpmX;
            eq &= s1.PhysPpmY == s2.PhysPpmY;
            return eq;
        }

        static void Main(string[] args)
        {
            /////////////////////////////////////////////////////////////
            // Copy the test png before processing it

            Directory.SetCurrentDirectory("../../..");
            string filePath1 = "monster-test.png";
            string filePath2 = "tmp.png";
            File.Copy(filePath1, filePath2, true);

            /////////////////////////////////////////////////////////////
            // Set the optimizing options (optional)

            PngOptimizerDll.POSettings settings = new PngOptimizerDll.POSettings();
            settings.BackupOldPngFiles = false;
            settings.KeepInterlacing = true;
            settings.IE6Compatible = true;
            settings.SkipAnimatedGifs = true;
            settings.KeepFileDate = true;

            settings.BkgdOption = PngOptimizerDll.POChunkOption.Force;
            settings.BkgdColor = System.Drawing.Color.FromArgb(50, 60, 70).ToArgb();

            settings.TextOption = PngOptimizerDll.POChunkOption.Force;
            settings.TextKeyword = "Author";
            settings.TextData = "Plopman";

            settings.PhysOption = PngOptimizerDll.POChunkOption.Force;
            settings.PhysPpmX = 11811;
            settings.PhysPpmY = 11811;

            if (!PngOptimizerDll.SetSettings(settings))
            {
                Console.WriteLine("SetSettings failed");
                return;
            }
            
            /////////////////////////////////////////////////////////////
            // Sanity check

            PngOptimizerDll.POSettings gotSettings;
            PngOptimizerDll.GetSettings(out gotSettings);
            if (!AreSettingsEqual(settings, gotSettings))
            {
                Console.WriteLine("DLL bad impl");
                return;
            }

            /////////////////////////////////////////////////////////////
            // Optimize file
            if (PngOptimizerDll.OptimizeFile("tmp.png"))
            {
                Console.WriteLine("OptimizeFile ok, before: "
                    + new FileInfo(filePath1).Length + " after: "
                    + new FileInfo(filePath2).Length);
            }
            else
            {
                Console.WriteLine("OptimizeFile fail, cause: " + PngOptimizerDll.GetLastErrorString());
            }

            /////////////////////////////////////////////////////////////
            // Optimize memory loaded file
            byte[] imageBuf = File.ReadAllBytes("monster-test.png");
            byte[] result = new byte[imageBuf.Length + 400000];
            int resultSize = 0;

            if (PngOptimizerDll.OptimizeFileMem(imageBuf, imageBuf.Length, result, result.Length, out resultSize))
            {
                Console.WriteLine("OptimizeFileMem ok, before: "
                    + imageBuf.Length + " after: "
                    + resultSize);
            }
            else
            {
                Console.WriteLine("OptimizeFileMem fail, cause: " + PngOptimizerDll.GetLastErrorString());
            }

            Console.WriteLine("-- done --");
        }
    }
}
